# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protobuf/sns_sqs_message.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protobuf/sns_sqs_message.proto',
  package='proto',
  syntax='proto3',
  serialized_pb=_b('\n\x1eprotobuf/sns_sqs_message.proto\x12\x05proto\"%\n\x07Service\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04uuid\x18\x02 \x01(\t\"\x99\x01\n\x08Metadata\x12\x14\n\x0cmessage_uuid\x18\x01 \x01(\t\x12\x18\n\x10protocol_version\x18\x02 \x01(\t\x12$\n\x1c\x63ompatible_protocol_versions\x18\x03 \x03(\t\x12\x11\n\ttimestamp\x18\x04 \x01(\x01\x12\r\n\x05topic\x18\x05 \x01(\t\x12\x15\n\rdata_encoding\x18\x06 \x01(\t\"a\n\rSNSSQSMessage\x12\x1f\n\x07service\x18\x01 \x01(\x0b\x32\x0e.proto.Service\x12!\n\x08metadata\x18\x02 \x01(\x0b\x32\x0f.proto.Metadata\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\tb\x06proto3')
)




_SERVICE = _descriptor.Descriptor(
  name='Service',
  full_name='proto.Service',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='proto.Service.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uuid', full_name='proto.Service.uuid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=78,
)


_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='proto.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message_uuid', full_name='proto.Metadata.message_uuid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protocol_version', full_name='proto.Metadata.protocol_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compatible_protocol_versions', full_name='proto.Metadata.compatible_protocol_versions', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='proto.Metadata.timestamp', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='topic', full_name='proto.Metadata.topic', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_encoding', full_name='proto.Metadata.data_encoding', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=234,
)


_SNSSQSMESSAGE = _descriptor.Descriptor(
  name='SNSSQSMessage',
  full_name='proto.SNSSQSMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='service', full_name='proto.SNSSQSMessage.service', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='proto.SNSSQSMessage.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='proto.SNSSQSMessage.data', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=236,
  serialized_end=333,
)

_SNSSQSMESSAGE.fields_by_name['service'].message_type = _SERVICE
_SNSSQSMESSAGE.fields_by_name['metadata'].message_type = _METADATA
DESCRIPTOR.message_types_by_name['Service'] = _SERVICE
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
DESCRIPTOR.message_types_by_name['SNSSQSMessage'] = _SNSSQSMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Service = _reflection.GeneratedProtocolMessageType('Service', (_message.Message,), dict(
  DESCRIPTOR = _SERVICE,
  __module__ = 'protobuf.sns_sqs_message_pb2'
  # @@protoc_insertion_point(class_scope:proto.Service)
  ))
_sym_db.RegisterMessage(Service)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), dict(
  DESCRIPTOR = _METADATA,
  __module__ = 'protobuf.sns_sqs_message_pb2'
  # @@protoc_insertion_point(class_scope:proto.Metadata)
  ))
_sym_db.RegisterMessage(Metadata)

SNSSQSMessage = _reflection.GeneratedProtocolMessageType('SNSSQSMessage', (_message.Message,), dict(
  DESCRIPTOR = _SNSSQSMESSAGE,
  __module__ = 'protobuf.sns_sqs_message_pb2'
  # @@protoc_insertion_point(class_scope:proto.SNSSQSMessage)
  ))
_sym_db.RegisterMessage(SNSSQSMessage)


# @@protoc_insertion_point(module_scope)
